package com.ashokit.ticketbooking.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.Customizer;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.provisioning.InMemoryUserDetailsManager;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;

//Our Entire InMemory Authentication and Authorization code we will write here

@Configuration
@EnableWebSecurity

/*
 * @EnableWebSecurity ==> Default Security Filter ==> WebsecurityConfigureAdapter
 * We are going to change the default behaviour of Security Filter with our own logic
 * Hence we need to write our own security filter withour own logic. 
 * @EnableWebSecurity ==> It will consider as SecurityConfig as default security filter and it loads
 * 						   based on the logic what ever we write
 */
public class SecurityConfig {
	
	
	//Authentication Logic ==> UserDetails Object with UserDetailsService contains this object
	
	//Rule1 ==> ashokitadmin/ashokitadmin with ADMIN Role
	//Rule2 ==> ashokitguest/ashokitguest with GUEST Role
	
	@Bean
	public UserDetailsService userDetailsService(PasswordEncoder encoder) {
	
	UserDetails adminObj=User.withUsername("ashokitadmin").password(encoder.encode("ashokitadmin")).
			roles("ADMIN").build();
	
	UserDetails guestObj=User.withUsername("ashokitguest").password(encoder.encode("ashokitguest")).
			roles("GUEST").build();
	
	
	return new InMemoryUserDetailsManager(adminObj,guestObj);
	
	
	}
	
	
	@Bean
	public PasswordEncoder passwordEncoder() {
		return new BCryptPasswordEncoder(); //Returning Base64 Password Encoder
	}
	
	//Authorization Logic
	
	//Rule1 ==> Admin Role Can acccess All Tickets and Individual Ticket
	//Rule2 ==> Guest Role can Access only Individual Ticket
	
	@Bean
	public SecurityFilterChain securityFilterChain(HttpSecurity authObj) throws Exception {
		
		//URL For All Tickets ==> http://localhost:8080/tickets/all
		
		//URL For Individual Ticket ==> localhost:8080/tickets/{passengerName}
		
		return authObj.authorizeHttpRequests(
				
				request -> request.requestMatchers(
				new AntPathRequestMatcher("/tickets/all/**")).hasAnyRole("ADMIN"))
				
				.authorizeHttpRequests(request -> request.requestMatchers(
						new AntPathRequestMatcher("/tickets/ticket/**")).hasAnyRole("ADMIN","GUEST")
						
						.anyRequest()
						.authenticated()).formLogin(Customizer.withDefaults()
						
						).build();
				
				
	}

}
